package insistesoft.multixnotes;


import android.content.Context;
import android.support.v7.app.AppCompatActivity;
import android.view.Gravity;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;

import insistesoft.multixnotes.beans.Student;
import insistesoft.multixnotes.beans.StudentsMarks;
import insistesoft.multixnotes.beans.Subject;
import insistesoft.multixnotes.beans.TypePercentQual;
import insistesoft.multixnotes.tools.FilesName;

public class GestorDades extends AppCompatActivity {

    /**
     * Funció que afegeix una assignatura al fitxer d'assignatures
     * @param subject assignatura a afegir
     */
    public void setSubjectsFile(Subject subject, Context ctx) {
        try {
            ArrayList<Subject> subjects = new ArrayList<>();
            OutputStreamWriter newSubject;
            if(checkFile(FilesName.FILESUBJECTS, ctx)) {
                subjects = getSubjectsFile(ctx);
            }
            subjects.add(subject);

            Collections.sort(subjects);

            newSubject = new OutputStreamWriter(ctx.openFileOutput(FilesName.FILESUBJECTS, Context.MODE_PRIVATE));
            for (Subject subj:subjects) {

                newSubject.write(subj.getIdSubject());
                newSubject.write(System.getProperty("line.separator"));
                newSubject.write(subj.getNameSubject());
                newSubject.write(System.getProperty("line.separator"));
            }

            newSubject.close();
        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }


    /**
     * Funció per crea i afegeix una llista d'assignatures al fitxer d'assignatures
     * @param subjects llista d'assignatures
     */
    public void setSubjectsFile(ArrayList<Subject> subjects, Context ctx) {
        try {
            Collections.sort(subjects);
            OutputStreamWriter newSubject = new OutputStreamWriter(ctx.openFileOutput(FilesName.FILESUBJECTS, Context.MODE_PRIVATE));

            for(Subject tempsubjects:subjects) {
                newSubject.write(tempsubjects.getIdSubject());
                newSubject.write(System.getProperty("line.separator"));
                newSubject.write(tempsubjects.getNameSubject());
                newSubject.write(System.getProperty("line.separator"));
            }
            newSubject.close();

        } catch (Exception ex) {

        }
    }

    /**
     * Funció que obté la llista d'assignatures d'un fitxer o una llista buida si el fitxer no existeix
     * @return una llista d'assignatures
     */
    public ArrayList<Subject> getSubjectsFile(Context ctx) {
        ArrayList<Subject> subjects = new ArrayList<>();
        Subject subject;

        try {
            if (checkFile(FilesName.FILESUBJECTS, ctx)) {
                BufferedReader brarxiu = new BufferedReader(new InputStreamReader(ctx.openFileInput(FilesName.FILESUBJECTS)));
                String subjectIdLine = brarxiu.readLine(), subjectNameLine;

                while (subjectIdLine != null) {
                    subjectNameLine = brarxiu.readLine();
                    subject = new Subject(subjectIdLine, subjectNameLine);
                    subjects.add(subject);
                    subjectIdLine = brarxiu.readLine();
                }
                brarxiu.close();
            } else {
                subject = new Subject("", "");
                subjects.add(subject);
            }
        } catch (IOException e) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_carregar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
        return subjects;
    }

    /**
     * Funció que afegeix un grup al fitxer de grups
     * @param group grup a afegir
     */
    public void setGroupsFile(String group, Context ctx) {
        try {
            OutputStreamWriter newgroup;
            if(checkFile(FilesName.FILEGROUPS, ctx)) {
                newgroup = new OutputStreamWriter(ctx.openFileOutput(FilesName.FILEGROUPS, Context.MODE_APPEND));
            } else {
                newgroup = new OutputStreamWriter(ctx.openFileOutput(FilesName.FILEGROUPS, Context.MODE_PRIVATE));
            }
            newgroup.write(group);
            newgroup.write(System.getProperty("line.separator"));

            newgroup.close();

        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }

    /**
     * Funció que obté una llista de grups d'un fitxer o una llista buida si el fitxer no existeix
     * @return una llista de grups
     */
    public ArrayList<String> getGroupsFile(Context ctx) {
        ArrayList<String> groups = new ArrayList<>();
        try {
            if (checkFile(FilesName.FILEGROUPS, ctx)) {
                BufferedReader brarxiu = new BufferedReader(new InputStreamReader(ctx.openFileInput(FilesName.FILEGROUPS)));
                String group = brarxiu.readLine();

                while (group != null) {

                    groups.add(group);
                    group = brarxiu.readLine();
                }
                brarxiu.close();
            } else {
                String errorGroup = "";
                groups.add(errorGroup);
            }
        } catch (IOException e) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_carregar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
        return groups;
    }

    /**
     * Funció que afegeix un estudiant al fitxer del seu grup
     * @param group grup de l'estudiant
     * @param student estudiant a afegir
     */
    public void setStudentsFile(String group, Student student, Context ctx) {
        try {
            OutputStreamWriter newstudent;
            if(checkFile(group+FilesName.NAMEGROUP, ctx)) {
                newstudent = new OutputStreamWriter(ctx.openFileOutput(group + FilesName.NAMEGROUP, Context.MODE_APPEND));
            } else {
                newstudent = new OutputStreamWriter(ctx.openFileOutput(group + FilesName.NAMEGROUP, Context.MODE_PRIVATE));
            }
            newstudent.write(student.getIdStudent());
            newstudent.write(System.getProperty("line.separator"));
            newstudent.write(student.getNameStudent());
            newstudent.write(System.getProperty("line.separator"));
            newstudent.close();
        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }

    }

    /**
     * Funció que crea i afegeix una llista d'estudiants al fitxer del seu grup
     * @param group grup dels estudiants
     * @param students llista d'estudiants
     */
    public void setStudentsFile(String group, ArrayList<Student> students, Context ctx) {
        try {

            OutputStreamWriter newstudent = new OutputStreamWriter(ctx.openFileOutput(group + FilesName.NAMEGROUP, Context.MODE_PRIVATE));

            for(Student tempstudent:students) {
                newstudent.write(tempstudent.getIdStudent());
                newstudent.write(System.getProperty("line.separator"));
                newstudent.write(tempstudent.getNameStudent());
                newstudent.write(System.getProperty("line.separator"));
            }
            newstudent.close();

        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }

    /**
     * Funció que obté una llista d'estudiants del fitxer del seu grup o una llista buida si no existeix el fitxer
     * @param group grup dels estudiants
     * @return una llista d'estudiants
     */
    public ArrayList<Student> getStudentsFile(String group, Context ctx) {
        ArrayList<Student> students = new ArrayList<>();
        Student student;

        try {
            if(checkFile(group + FilesName.NAMEGROUP, ctx)) {
                BufferedReader brarxiu = new BufferedReader(new InputStreamReader(ctx.openFileInput(group + FilesName.NAMEGROUP)));
                String studentIdLine = brarxiu.readLine(), studentNameLine;

                while (studentIdLine != null) {
                    studentNameLine = brarxiu.readLine();
                    student = new Student(studentIdLine, studentNameLine);
                    students.add(student);
                    studentIdLine = brarxiu.readLine();
                }
                brarxiu.close();
            } else {
                student = new Student("", "");
                students.add(student);
            }
        } catch (IOException e) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_carregar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
        return students;
    }

    /**
     * Funció que crea i afegeix una llista d'estudiants i les seves notes a l'arxiu d'una assignatura i del mètode de qualificació
     * @param idSubject l'identificador de l'assignatura
     * @param group el grup dels estudiants
     * @param typeQ mètode de qualificació
     * @param studentsMarks Llista d'estudians i notes
     */
    public void setStudentsMarksFile(String idSubject, String group, String typeQ, ArrayList<StudentsMarks> studentsMarks, Context ctx) {
        try {
            OutputStreamWriter newsubject= new OutputStreamWriter(ctx.openFileOutput(idSubject+group+typeQ+FilesName.EXTENSION, Context.MODE_PRIVATE));

            for (StudentsMarks studentMark: studentsMarks) {
                newsubject.write(studentMark.getStudent());
                newsubject.write(System.getProperty("line.separator"));
                newsubject.write(Float.toString(studentMark.getMark()));
                newsubject.write(System.getProperty("line.separator"));
            }
            newsubject.close();

        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }

    /**
     * Funció que obté una llista d'estudiants i notes per a una assignatura i un mètode de qualificació
     * @param idSubject identificador de l'assignatura
     * @param group grup dels estudiants
     * @param typeQ mètode de qualificació
     * @return una llista d'estudiants i notes
     */
    public ArrayList<StudentsMarks> getStudentsMarksFile(String idSubject, String group, String typeQ, Context ctx) {
        ArrayList<StudentsMarks> studentsMarks = new ArrayList<>();
        StudentsMarks studentMark;

        try {
            if (checkFile(idSubject+group+typeQ+FilesName.EXTENSION, ctx)) {
                BufferedReader brarxiu = new BufferedReader(new InputStreamReader(ctx.openFileInput(idSubject + group + typeQ + FilesName.EXTENSION)));
                String studentMarkIdLine = brarxiu.readLine(), studentMarkQuaLine;

                while (studentMarkIdLine != null) {
                    studentMarkQuaLine = brarxiu.readLine();
                    studentMark = new StudentsMarks(studentMarkIdLine, Float.valueOf(studentMarkQuaLine));
                    studentsMarks.add(studentMark);
                    studentMarkIdLine = brarxiu.readLine();
                }
                brarxiu.close();
            } else {
                studentMark = new StudentsMarks("", 0);
                studentsMarks.add(studentMark);
            }
        } catch (IOException e) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_carregar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }

        return studentsMarks;
    }

    /**
     * Funció que afegeix un mètode de qualificació i el seu percentatge a una assignatura i grup d'estudiants
     * @param idSubject identificador de l'assignatura
     * @param group grup dels estudiants
     * @param typePercentQual mètode de qualificació i percentatge sobre la nota final
     */
    public void setTypePercentQualFile(String idSubject, String group, TypePercentQual typePercentQual, Context ctx) {
        try {
            OutputStreamWriter newType;
            if(checkFile(idSubject+group+FilesName.NAMETYPE, ctx)) {
                newType = new OutputStreamWriter(ctx.openFileOutput(idSubject+group+FilesName.NAMETYPE, Context.MODE_APPEND));
            } else {
                newType = new OutputStreamWriter(ctx.openFileOutput(idSubject + group + FilesName.NAMETYPE, Context.MODE_PRIVATE));
            }
            newType.write(typePercentQual.getTypeQual());
            newType.write(System.getProperty("line.separator"));
            newType.write(Integer.toString(typePercentQual.getPercentQual()));
            newType.write(System.getProperty("line.separator"));

            newType.close();
        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }

    /**
     * Funció que modificac un percentatge per a un mètode de qualificació al fitxer duna assignatura i grup
     * @param idSubject identificador de l'assignatura
     * @param group el grup dels estudiants
     * @param type mètode de qualificació
     * @param percent percentatge sobre la nota final
     */
    public void setTypePercentQualFile(String idSubject, String group, String type, String percent, Context ctx) {
        try {
            ArrayList<TypePercentQual> typePercentQuals = getTypeQualFile(idSubject, group, ctx);
            OutputStreamWriter newType = new OutputStreamWriter(ctx.openFileOutput(idSubject + group + FilesName.NAMETYPE, Context.MODE_PRIVATE));

            for (TypePercentQual typePQ : typePercentQuals) {

                if (typePQ.getTypeQual().equals(type)) {
                    newType.write(type);
                    newType.write(System.getProperty("line.separator"));
                    newType.write(percent);
                    newType.write(System.getProperty("line.separator"));
                } else {
                    newType.write(typePQ.getTypeQual());
                    newType.write(System.getProperty("line.separator"));
                    newType.write(Integer.toString(typePQ.getPercentQual()));
                    newType.write(System.getProperty("line.separator"));
                }
            }

            newType.close();
        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }

    /**
     * Funció que obté un mètode i percentatge per a un grup i assignatura
     * @param idSubject identificador de l'assignatura
     * @param group grup d'estudiants
     * @return una llista de mètodes d'evaluació i el percentatge sobre la nota final
     */
    public ArrayList<TypePercentQual> getTypeQualFile(String idSubject, String group, Context ctx) {
        ArrayList<TypePercentQual> typesPercentQual = new ArrayList<>();
        TypePercentQual typePercentQual;

        try {
            if (checkFile(idSubject+group+FilesName.NAMETYPE, ctx)) {
                BufferedReader brarxiu = new BufferedReader(new InputStreamReader(ctx.openFileInput(idSubject + group + FilesName.NAMETYPE)));
                String typeNameLine = brarxiu.readLine(), typePercentLine;

                while (typeNameLine != null) {
                    typePercentLine = brarxiu.readLine();
                    typePercentQual = new TypePercentQual(typeNameLine, Integer.valueOf(typePercentLine));
                    typesPercentQual.add(typePercentQual);
                    typeNameLine = brarxiu.readLine();
                }
                brarxiu.close();
            }
        } catch (IOException e) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_carregar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
        return typesPercentQual;
    }

    /**
     * Funció que afegeix un nou mètode d'avaluació a la llista particular de l'usuari
     * @param userType nou mètode d'avaluació
     */
    public void setUserTypesFile(String userType, Context ctx) {

        try {
            OutputStreamWriter newUserType;
            if(checkFile(FilesName.FILEUSERTYPES, ctx)) {
                newUserType = new OutputStreamWriter(ctx.openFileOutput(FilesName.FILEUSERTYPES, Context.MODE_APPEND));
            } else {
                newUserType = new OutputStreamWriter(ctx.openFileOutput(FilesName.FILEUSERTYPES, Context.MODE_PRIVATE));
            }
            newUserType.write(userType);
            newUserType.write(System.getProperty("line.separator"));

            newUserType.close();
        } catch (Exception ex) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_grabar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
    }

    /**
     * Funció que obté la llista dels mètodes d'avaluació generats per l'usuari
     * @return la llista de mètodes d'avaluació de l'usuari
     */
    public ArrayList<String> getUserTypesFile(Context ctx) {
        ArrayList<String> userTypes = new ArrayList<>();
        try {
            if (checkFile(FilesName.FILEUSERTYPES, ctx)) {
                BufferedReader brarxiu = new BufferedReader(new InputStreamReader(ctx.openFileInput(FilesName.FILEUSERTYPES)));
                String userTypeLine = brarxiu.readLine();

                while (userTypeLine != null) {
                    userTypes.add(userTypeLine);
                    userTypeLine = brarxiu.readLine();
                }
                brarxiu.close();
            } else {
                userTypes.add("");
            }
        } catch (IOException e) {
            Toast toastIC = Toast.makeText(getApplicationContext(), getResources().getString(R.string.error_carregar_arxiu), Toast.LENGTH_LONG);
            toastIC.setGravity(Gravity.CENTER,0,0);
            toastIC.show();
        }
        return userTypes;
    }

    /**
     * Funció auxiliar que comprova si un fitxer existeix
     * @param arxiu nom de l'arxiu a comprovar
     * @return true si existeix
     *          false altrament
     */
    public boolean checkFile(String arxiu, Context ctx) {

        String[] llistaarxius = ctx.fileList();
        for (String llistaarxiu : llistaarxius) {
            if (arxiu.equals(llistaarxiu)) {
                return true;
            }
        }
        return false;
    }
}
