package insistesoft.multixnotes;

import android.content.Context;

import java.util.ArrayList;

import insistesoft.multixnotes.beans.Student;
import insistesoft.multixnotes.beans.StudentsMarks;
import insistesoft.multixnotes.beans.Subject;
import insistesoft.multixnotes.beans.TypePercentQual;

/**
 *
 * Interface to specify user's operations
 */

public interface GestorMxNInterface {

    /**
     * Function to create a new subject
     *
     * @param idSubject subject's ID
     * @param nameSubject subject's name
     */
    public boolean addSubject(String idSubject, String nameSubject, Context ctx);

    /**
     * Function to add a new student
     *
     * @param student student's ID
     * @param nameStudent student's name
     * @param group student's group
     */
    public boolean addStudent(String student, String nameStudent, String group, Context ctx);

    /**
     * Function to add a new qualification method
     *
     * @param nameQual qualification method
     */
    public boolean addTypeQualification(String nameQual, Context ctx);

    /**
     * Function to add the qualifications of a group and subject
     *
     * @param idSubject subject's ID
     * @param group student's group
     * @param typeQ qualification method
     * @param studentsMarks Students' qualifications
     */
    public void addQualifications(String idSubject, String group, String typeQ, ArrayList<StudentsMarks> studentsMarks, Context ctx);

    /**
     * Function to indicate the value (percentage) that has a subject for a group
     *
     * @param subject subject ID
     * @param group student's group
     * @param type qualification method
     * @param percent percentage
     */
    public void addTypePercentQual(String subject, String group, String type, String percent, Context ctx);

    /**
     *  Function to modify a student
     *
     * @param group students' groups
     * @param student students' list
     */
    public void modifyStudent(String group, ArrayList<Student> student, Context ctx);

    /**
     * Function to modify a subject
     *
     * @param subjects subjects' list
     */
    public void modifySubject(ArrayList<Subject> subjects, Context ctx);

    /**
     * Function to modify the percentage of a qualification method
     *
     * @param subject subjec's ID
     * @param group student's group
     * @param type qualification method
     * @param percent percentage
     */
    public void modifyTypePercentQual(String subject, String group, String type, String percent, Context ctx);

    /**
     * Function to get a subject's list
     *
     * @return subject's list
     */
    public ArrayList<Subject> getSubjects(Context ctx);

    /**
     * Function to get the groups
     *
     * @return groups' list
     */
    public ArrayList<String> getGroups(Context ctx);

    /**
     * Function to get a students' list
     * @param group students' group
     * @return students' list
     */
    public ArrayList<Student> getStudents(String group, Context ctx);

    /**
     * Function to get the students' qualifications
     *
     * @param subject subject ID
     * @param group students' group
     * @param typeQ qualification method
     *
     * @return students' qualifications list
     */
    public ArrayList<StudentsMarks> getStudentsMarks(String subject, String group, String typeQ, Context ctx);

    /**
     * Function to get the value (percentage) that has a subject for a group
     * @param subject subject ID
     * @param group students' group
     *
     * @return qualification method and percentage list
     */
    public ArrayList<TypePercentQual> getTypePercentQual(String subject, String group, Context ctx);

    /**
     * Function to get user qualification method
     *
     * @return the user qualification method list
     */
    public ArrayList<String> getUserTypes(Context ctx);

    /**
     * Function to choose the first screen to show. Following the normal introduction data.
     *
     * @return  1 add subjects screen
     *          2 add students screen
     *          3 add qualifications screen
     */
    public int firstScreen(Context ctx);
}
